import httpx
from typing import Dict
from tradebots.tradebot import TradeBotInput, TradeBotOutput
from datetime import datetime

class TradeBotInterface:
    '''
    Interface to interact with TradeBotServer via HTTP API
    '''
    def __init__(self, host: str = "0.0.0.0", port: int = 8000):
        '''
        Args:
        - host (str): Host where the TradeBotServer is running
        - port (int): Port where the TradeBotServer is listening
        '''
        self.base_url = f"http://{host}:{port}"

    def reset(self) -> Dict[str, str]:
        '''
        Sends a reset request to the TradeBotServer

        Returns:
        - Dict[str, str]: Response from the server
        '''
        response = httpx.post(f"{self.base_url}/reset")
        response.raise_for_status()  # Raises an exception if the request was unsuccessful
        return response.json()

    def act(self, input: TradeBotInput) -> TradeBotOutput:
        '''
        Sends a request to act on the input data

        Args:
        - input (TradeBotInput): The input data for the TradeBot

        Returns:
        - TradeBotOutput: The output data from the TradeBot
        '''
        if isinstance(input.timestamp, datetime):
            input.timestamp = input.timestamp.isoformat()
        response = httpx.post(f"{self.base_url}/act", json=input.dict())
        response.raise_for_status()  # Raises an exception if the request was unsuccessful
        return TradeBotOutput(**response.json())


from tradebots.tradebot import TradeBotInput, TradeBotOutput, TradeBot
from tradebots.tradebot import DullBot
from tradebots.server import TradeBotServer


app = TradeBotServer(DullBot())
app.listen()


from fastapi import FastAPI
from fastapi.responses import JSONResponse
from tradebots.tradebot import TradeBotInput, TradeBotOutput, TradeBot


class TradeBotServer():
    '''
    Wrapper for TradeBot to expose it as a REST API
    '''
    def __init__(self, bot: TradeBot):
        '''
        Args:
        - bot (TradeBot): TradeBot instance
        '''
        self.bot = bot
        self.app = FastAPI()

        # Route for the reset method
        @self.app.post("/reset")
        def reset():
            self.bot.reset()
            return JSONResponse(content={"status": "success", "message": "Bot reset successfully"})

        # Route for the act method
        @self.app.post("/act", response_model=TradeBotOutput)
        def act(input: TradeBotInput):
            output = self.bot.act(input)
            return output

    def listen(self, host: str = "0.0.0.0", port: int = 8000):
        import uvicorn
        uvicorn.run(self.app, host=host, port=port)



from pydantic import BaseModel
from datetime import datetime
from typing import List
from enum import Enum
from abc import ABC, abstractmethod


class OrderAction(str, Enum):
    BUY = 'BUY'
    SELL = 'SELL'
    HOLD = 'HOLD'


class Stock(BaseModel):
    ticker: str
    open: float
    high: float
    low: float
    close: float


class TradeBotInput(BaseModel):
    stocks: List[Stock]
    timestamp: datetime | str
    platform_id: str


class Order(BaseModel):
    ticker: str
    price: float
    timestamp: datetime | str
    platform_id: str
    action: OrderAction


class TradeBotOutput(BaseModel):
    orders: List[Order]
    timestamp: datetime
    platform_id: str


class TradeBot(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def reset(self):
        '''
        description: reset tradebot to initial state
        '''
        pass

    @abstractmethod
    def act(self, input: TradeBotInput) -> TradeBotOutput:
        '''
        args:
            input: TradeBotInput - input data for tradebot
        returns:
            TradeBotOutput - output data for tradebot
        description: base class for tradebots
        '''
        pass


class DullBot(TradeBot):
    '''
    DullBot - tradebot that buys and sells all stocks in the same timestamp.
    Only for demonstration purposes.
    '''
    def __init__(self):
        super(TradeBot).__init__()
        self.current_order = OrderAction.BUY

    def reset(self):
        self.current_order = OrderAction.BUY

    def act(self, input: TradeBotInput) -> TradeBotOutput:
        orders = []

        if isinstance(input.timestamp, datetime):
            input.timestamp = input.timestamp.isoformat()

        for stock in input.stocks:
            orders.append(Order(ticker=stock.ticker,
                                price=stock.close,
                                timestamp=input.timestamp.isoformat(),
                                platform_id=input.platform_id,
                                action=self.current_order))
        if self.current_order == OrderAction.BUY:
            self.current_order = OrderAction.SELL
        else:
            self.current_order = OrderAction.BUY
        return TradeBotOutput(orders, input.timestamp, input.platform_id)



from .tradebot import TradeBot, DullBot, TradeBotInput, TradeBotOutput
from .server import TradeBotServer
from .interface import TradeBotInterface


__all__ = [
    "TradeBot",
    "DullBot",
    "TradeBotInput",
    "TradeBotOutput",
    "TradeBotServer",
    "TradeBotInterface"
]


from setuptools import setup, find_packages

setup(
    name="tradebots",
    version="0.5.15",
    author="Kryzhanovskiy Maxim",
    author_email="kryzhanovskiymax@mail.ru",
    description="Package for building tradebots",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/yourusername/your_package",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'typing==3.7.4.3',
        "requests==2.32.3",
        "fastapi==0.112.0",
        "httpx==0.20.0"
    ]
)